import { Setting, MarkStore, Track } from "@/types";
import { reactive, watch } from "vue";

const obj = {
  setting: {
    mode: "TRACK",
    examType: "SCAN_IMAGE",
    forceMode: false,
    sheetView: false,
    sheetConfig: [],
    enableAllZero: false,
    fileServer: "",
    userName: "",
    subject: <Setting["subject"]>{},
    forceSpecialTag: false,
    uiSetting: {
      "answer.paper.scale": 1,
      "score.board.collapse": false,
      "normal.mode": "keyboard",
      "paper.modal": false,
      "answer.modal": false,
      "minimap.modal": false,
      "specialTag.modal": false,
      "score.fontSize.scale": 1,
    },
    statusValue: "FORMAL",
    problemTypes: [],
    groupNumber: 0,
    groupTitle: "",
    topCount: 0,
    splitConfig: [],
    prefetchCount: 3,
    startTime: 0,
    endTime: 0,
    selective: false,
  },
  status: <MarkStore["status"]>{},
  groups: [],
  tasks: [],
  message: null,
  currentTask: undefined,
  currentQuestion: undefined,
  currentScore: undefined,
  currentSpecialTag: undefined,
  historyOpen: false,
  historyTasks: [],
  removeScoreTracks: [],
  focusTracks: [],
  maxModalZIndex: 1020,
  minimapScrollTo: 0,
  allPaperModal: false,
  sheetViewModal: false,
  globalMask: false,
} as MarkStore;

/**
 * 返回正在评卷的状态 '' | 回评 | 打回
 */
export function getMarkStatus() {
  if (!store.currentTask) return "";

  if (store.currentTask.previous) return "回评";
  if (store.currentTask.rejected) return "打回";

  return "";
}

/**
 * 是否是扫描阅卷
 * @returns boolean
 */
export function isScanImage() {
  return store.setting.examType === "SCAN_IMAGE";
}

export const store = reactive(obj);

watch(
  () => store.currentTask,
  () => {
    // 初始化 task.markResult ，始终保证 task 下有 markResult
    // 1. 评卷时，如果没有 markResult ，则初始化一个 markResult 给它
    // 1. 回评时，先清空它的 markResult ，然后初始化一个 markResult 给它
    if (!store.currentTask) return;

    const task = store.currentTask;
    if (task.previous && task.markResult) {
      // @ts-ignore 将其强制置空
      task.markResult = undefined;
    }
    if (!task.markResult) {
      const statusValue = store.setting.statusValue;
      const { libraryId, studentId, previous } = task;
      task.markResult = {
        statusValue: statusValue,
        libraryId: libraryId,
        studentId: studentId,
        spent: 0,

        trackList: task.questionList.reduce(
          (all, c) => all.concat(c.trackList),
          [] as Array<Track>
        ),
        specialTagList: [...(task.specialTagList ?? [])],
        scoreList: task.questionList.map((q) => q.score),
        markerScore: 0, // 后期通过 scoreList 自动更新

        problem: false,
        problemTypeId: 0,
        unselective: false,

        __markStartTime: Date.now(),
        __previous: previous,
      };
    }
  }
);
